/*
	Copyright (c) 2001 Maurice Gittens <maurce@gittens.nl>

   This software is distributed under the GNU General
   Public License.
	See the file COPYING for more information on usage and
	redistribution of this file and for a DISCLAIMER OF ALL
	WARRANTIES.

	Written by Maurice Gittens <maurice@gittens.nl>.
*/

#ifndef TINY_GRAMMARSYMBOL_SEEN
#define TINY_GRAMMARSYMBOL_SEEN

#include <string>
#include <tiny_util.h>

//
//
//
class tiny_grammarSymbol
{
public:
   enum Type
   {
      reserverWord = 1,
      terminal,
      nonterminal,
      action,
      generic,
      variable,
   };

	tiny_grammarSymbol(Type t,
	                   const std::string& n,
	                   unsigned int i,
							 unsigned lineno)
		: id(i), type(t), name(n), used(false), lineNo(lineno)
	{
		used = 0;
	}

	virtual ~tiny_grammarSymbol() {}
	
	const std::string& getName() const { return name; }
	unsigned int getId() const { return id; }
	virtual unsigned int getEncodedId() const { return makeTerminal(id); }
	bool isUsed() const { return used; }
	void setUsed() { used = 1; }
	unsigned int getLineNo() const { return lineNo; }
	void setLineNo(int l) { lineNo = l; }
   Type getType() const { return type; }
	
protected:	
	unsigned int id;

private:
	Type type;
	std::string name;
	bool used;
	unsigned int lineNo;
};

typedef tiny_grammarSymbol tiny_ReservedWord;
typedef tiny_grammarSymbol tiny_Variable;


#endif
