/*
	Copyright (c) 2001 Maurice Gittens <maurce@gittens.nl>

   This software is distributed under the GNU General
   Public License.
	See the file COPYING for more information on usage and
	redistribution of this file and for a DISCLAIMER OF ALL
	WARRANTIES.

	Written by Maurice Gittens <maurice@gittens.nl>.
*/

#ifndef TINY_PARSER_SEEN
#define TINY_PARSER_SEEN

#include <string>
#include <vector>
#include <tiny_lexer.h>
#include <tiny_symbolstring.h>
#include <tiny_grammarsymbol.h>

class tiny_lexer;
class tiny_principle;

/**
 *  simple recursive descent parser the tiny grammar file
 */
class tiny_parser
{
public:
   tiny_parser(tiny_lexer& l)
	: lexer(l)
	{
	}

   void operator()();

private:
   tiny_lexer& lexer;
   int lookAhead;

   void fileDeclarationList();
   void fileDeclaration();
   void grammarHead();
   void grammarDeclSectionList();
   void grammarDeclSection();
   void principlesSection();
   void genericSymbolDefinition();
   void symbolDefinition();
   bool symbolDefLhs(tiny_principle&, tiny_rewritableSymbol**, bool isGeneric = false);
   void symbolDeclSection(tiny_grammarSymbol::Type);
   void tiny_parser::principleRhsList(tiny_principle&);
   void tiny_parser::principleRhs(tiny_principle&);
   void symbolList(tiny_symbolString &);
   bool parmList();

   bool nextIs(int t)
   {
      if (t == (lookAhead = lexer.getToken()))
		   return true;
   	else
	   {
	      lexer.error(std::string("syntax error: read '") +
	                  lexer.getTokenClass(lookAhead)
		               + "' when '" + lexer.getTokenClass(t)
				   		+ "' was expected");
         advance();
         return false;
      }
   }

   bool currentIs(int t)
   {
      if (t == lookAhead)
      {
         advance();
		   return true;
		}
   	else
	   {
	      lexer.error(std::string("syntax error: read '") +
	                  lexer.getTokenClass(lookAhead)
		               + "' when '" + lexer.getTokenClass(t)
				   		+ "' was expected");
         advance();
         return false;
      }
   }

   void advance()
   {
      lookAhead = lexer.getToken();
   }

   void endGenericSymbolDefinition();
   void endSymbolDefinition();
   void endGrammarRules();
   void endFileDeclaration();
   void endGrammarDeclaration();
   void endPrinciplesSection();
   void substituteVariables(tiny_principle&, const std::vector<std::string>& args);
   void substituteVariables(tiny_symbolString&, const std::vector<std::string>& args);

};

#endif
