/*
	Copyright (c) 2001 Maurice Gittens <maurce@gittens.nl>

   This software is distributed under the GNU General
   Public License.
	See the file COPYING for more information on usage and
	redistribution of this file and for a DISCLAIMER OF ALL
	WARRANTIES.

	Written by Maurice Gittens <maurice@gittens.nl>.
*/

#include <tiny_principle.h>
#include <vector>

using namespace std;

tiny_principle::tiny_principle()
: offset(0)
{
   used = false;
}

tiny_principle::~tiny_principle()
{
}

static bool empty(const vector<bool>& v)
{
   const unsigned size = v.size();

   unsigned int j = 0;
   bool rv = true;
   for(; j < size; ++j)
   {
      if (v[j])
         rv = false;
   }

   return rv;
}

static bool disjoint (const std::vector<bool>& s1, const std::vector<bool>& s2)
{
   assert( s1.capacity() == s2.capacity() );
   assert( s1.size() == s2.size() );

   unsigned int size = s1.size() > s2.size() ? s1.size() : s2.size();

   for(unsigned int i = 0; i < size; ++i)
   {
     if (s1[i] && s2[i])
        return false;
   }

   return true;
}

std::vector<bool>& operator|=(std::vector<bool>& d,
                              const std::vector<bool>& s)
{
   assert( d.capacity() == s.capacity() );
   assert( d.size() == s.size() );

   unsigned int size = d.size() > s.size() ? d.size() : s.size();

   for(unsigned int i = 0; i < size; ++i)
   {
      if (s[i])
         d[i] = s[i];
   }

   return d;
}



int tiny_principle::getDefaultRHSCount() const
{
	const int altCount = getAlternativeCount();
	int actionCount = 0;
	int rv = 0;

	for(int i = 0; i < altCount; i++)
	{
		const RightHandSide &rhs = getRHS(i);

		if (rhs.hasSelectionAction())
		{
			actionCount++;
			continue;
		}

		if (!empty(rhs.getFirstSet()))
         continue;
         		
		rv++;
	}

	if (actionCount == altCount - 1)
		return 1;
	else
		return rv;
}

//
//
//
void tiny_principle::updateFirstSets(bool& didS)
{
	const int altCount = getAlternativeCount();

   for(int i = 0; i < altCount; i++)
	{
		RightHandSide &rhs = getRHS(i);
		tiny_principle *rule = rhs.getRule();

		if (rule)
		{
			if (rhs.getFirstSet() != rule->getFirstSet())
			{
			   if (rule != this)
			   {
				   rhs.getFirstSet() = rule->getFirstSet();
               firstSet |= rhs.getFirstSet();
               didS = true;
	   		}
			}
		}
	}

}

bool tiny_principle::areRHS_Disjoint() const
{
   const int altCount = getAlternativeCount();
   std::vector<bool> auxSet(firstSet.size(), false);

   for(int i = 0; i < altCount; ++i)
   {
	   const std::vector<bool>& fs = getRHS(i).getFirstSet();
	
      if (!disjoint(auxSet,fs))
		   return false;

      auxSet |= fs;
	}

   return true;
}

#if 0
bool tiny_principle::areRHS_accesible() const
{
   const int altCount = getAlternativeCount();
   for(int i = 0; i < altCount; ++i)
   {
      const RightHandSide& rhs = getRHS(i);

      if (rhs.hasSelectionAction())
         continue;

      if (empty(rhs.getFirstSet()))
         return false;
	}

   return true;
}
#endif

