/*
	Copyright (c) 2001 Maurice Gittens <maurce@gittens.nl>

   This software is distributed under the GNU General
   Public License.
	See the file COPYING for more information on usage and
	redistribution of this file and for a DISCLAIMER OF ALL
	WARRANTIES.

	Written by Maurice Gittens <maurice@gittens.nl>.
*/

#ifndef TINY_PRINCIPLE_SEEN
#define TINY_PRINCIPLE_SEEN

#include <tiny_symbolstring.h>
#include <vector>

//
//
//
class tiny_principle
{
public:

	//
	//
	//
	class RightHandSide
	{
	public:
		RightHandSide() { leaf = offset = 0; next = 0; };

		unsigned int getOffset() const { return offset; }
		void setOffset(int o) { offset = o; }

		RightHandSide& push_back(int i) { str.push_back(i); return *this; }
		tiny_symbolString& getString() { return str; }
      tiny_symbolString& getStringToRecord() { return str2Rec; }
		void setString(const tiny_symbolString& s) { str = s; }

		int length() const { return str.length(); }

		std::vector<bool>& getFirstSet() { return firstSet; }
		const std::vector<bool>& getFirstSet() const { return firstSet; }
		int isNullable() const { return str.isNullable(); }
		int isLeaf() const { return leaf; }
		void setLeaf(int a = 1) { leaf = a; }
		int hasSelectionAction() const {  return isAction(str.getSymbol(0)); }

		tiny_principle *getRule() const { return next; }
		void setRule(tiny_principle *r) { next = r; }

      void setFirstSetCapacity(unsigned int s)
      {
         firstSet.reserve(s);

         for(unsigned i = 0; i < s; ++i)
            firstSet.push_back(false);

      }

	private:
		int offset;
      tiny_symbolString str2Rec;
		tiny_symbolString str;
		std::vector<bool> firstSet;
		tiny_principle *next;
		int leaf;
	};

	tiny_principle();
	~tiny_principle();

	const tiny_symbolString& getLHS() const { return lhs; }
	tiny_symbolString& getLHS() { return lhs; }
	RightHandSide& getRHS(int i) { return rhs[i]; }
	const RightHandSide& getRHS(int i) const { return rhs[i]; }

	int getDefaultRHSCount() const;
	RightHandSide& getLastRHS() { return getRHS(getAlternativeCount() - 1);  }
		
	void setId(int i) { id = i; }
	int getId() const { return id; }

	void setLHS(const tiny_symbolString& s) { lhs = s; }
	void setRHS(const tiny_symbolString& s)
	{
		RightHandSide r;

		rhs.push_back(r);
		rhs.back().setString(s);
	}

	int getAlternativeCount() const { return rhs.size(); }

	unsigned int getOffset() const { return offset; }
	void setOffset(int o) { offset = o; }

	const std::vector<bool>& getFirstSet() const { return firstSet; }
	std::vector<bool>& getFirstSet() { return firstSet; }
	void updateFirstSets(bool&);

	void setUsed() { used = true; }
	bool getUsed() const { return used; }

	int isSingleton() const { return getAlternativeCount() == 1; }

	bool areRHS_Disjoint() const;
	bool areRHS_accesible() const;

	RightHandSide& newRHS() { rhs.push_back(RightHandSide()); return rhs.back(); }

   void setFirstSetCapacity(unsigned int s)
   {
      firstSet.reserve(s);

      for(unsigned i = 0; i < s; ++i)
         firstSet.push_back(false);

      for(std::vector<RightHandSide>::iterator it = rhs.begin();
          it != rhs.end();
          ++it)
      {
         it->setFirstSetCapacity(s);
      }
   }

private:
	std::vector<RightHandSide> rhs;
	std::vector<bool> firstSet;
	tiny_symbolString lhs;

	unsigned int id;
	unsigned int offset;

	bool used;
};

#endif
