#ifndef TINY_SYM_SEEN
#define TINY_SYM_SEEN

#include <tiny_rule.h>
#include <string>

//
//
//
class GrammarSymbol
{
public:
	GrammarSymbol(const string& n, unsigned int i, unsigned lineno)
		: name(n), id(i), used(false), lineNo(lineno)
	{
		used = 0;
	}

	const string& getName() const { return name; }
	unsigned int getId() const { return id; }
	bool isUsed() const { return used; }
	GrammarSymbol& setUsed() { used = 1; return *this; }
	unsigned int getLineNo() const { return lineNo; }
	GrammarSymbol& setLineNo(int l) { lineNo = l; return *this; }

private:
	string name;
	unsigned int id;
	bool used;
	unsigned int lineNo;
};

typedef GrammarSymbol Terminal;
typedef GrammarSymbol GlobalTerminal;
typedef GrammarSymbol ReservedWord;
typedef GrammarSymbol Variable;

class NonTerminal : public GrammarSymbol
{
public:
	NonTerminal(const string& n, unsigned int i, unsigned ln) : GrammarSymbol(n,i,ln), offset(0)
	{
	}

	NonTerminal& addRule()
	{
		Rule r; rules.push_back(r);
		rules.back().setId(rules.size());
		return *this;
	}
	
	unsigned int getRuleCount() const { return rules.size(); }
	Rule &getRule(int i) { return rules[i]; }
	vector<Rule>& getRules() { return rules; }
	SymbolString& getLHS(int x) { return rules[x].getLHS(); }
	SymbolString& getRHS(int x, int y) { return rules[x].getRHS(y).getString(); }

	void setLHS(const SymbolString &s) { rules.back().setLHS(s); }
	void setRHS(const SymbolString &s) { rules.back().setRHS(s); }

	unsigned int getOffset() const { return offset; }
	void setOffset(int o) { offset = o; }


	int SameLhs();  // this function is intended to return
			// if to left hand sides are found to be
			// identical

private:
	vector<Rule> rules;
	unsigned offset;
};

#endif
