/*
	Copyright (c) 2001 Maurice Gittens <maurce@gittens.nl>

   This software is distributed under the GNU General
   Public License.
	See the file COPYING for more information on usage and
	redistribution of this file and for a DISCLAIMER OF ALL
	WARRANTIES.

	Written by Maurice Gittens <maurice@gittens.nl>.
*/

#ifndef TINY_SYMBOLPROCESSOR_SEEN
#define TINY_SYMBOLPROCESSOR_SEEN

#include <tiny_rewritablesymbol.h>
#include <tiny_principle.h>

class tiny_symbolString;
class tiny_action;
class tiny_rewritableSymbol;
class tiny_lexer;

//
//
//
class tiny_symbolProcessor
{
public:
   tiny_symbolProcessor(tiny_lexer& l)
   : scanner(l), checkTerminals(true), sortRules(true), conflict(false)
   {
   }

   void operator()() { processData(); }

	void processData();
	void printSymbolTable();
	void printActions();
	void printSymbolString(tiny_symbolString& s);

	void setSortRules(bool sr = true) { sortRules = sr; }
	
protected:
	void traverse(void (tiny_symbolProcessor::*)(tiny_action&));
	void traverse(void (tiny_symbolProcessor::*)(tiny_rewritableSymbol&));

	void printSymbols(tiny_rewritableSymbol&);
	void printSymbol(int id);

	void printAction(tiny_action&);

	void checkUsage();

	void doSortSymbolRules();
	void sortSymbolRules(tiny_rewritableSymbol&);

	void doReserveMemoryForFirstSets();
	void reserveMemoryForFirstSets(tiny_rewritableSymbol&);

	void doComputeInitialFirstSets();
	void computeInitialFirstSet(tiny_rewritableSymbol&);

	void computeFirstSets();

	void doUpdateFirstSets();
	void updateFirstSets(tiny_rewritableSymbol&);

	void doComputeConnections();
	void computeConnections(tiny_rewritableSymbol&);
	void connectRHS(tiny_principle::RightHandSide&);

	void doComputeSelectionSets();
	void computeSelectionSets(tiny_rewritableSymbol&);

private:
   tiny_lexer& scanner;
	bool checkTerminals;
	bool sortRules;
	bool didSomething;
	bool conflict;

};

#endif
